from .http_client import HttpClient

# Assign all the api classes
from .api.access_token import AccessToken
from .api.icd9 import Icd9
from .api.icd10 import Icd10


class Client(object):

    def __init__(self, auth={}, options={}):
        self.http_client = HttpClient(auth, options)

    def access_token(self):
        """Retrieve access token using API credentials.
        """
        return AccessToken(self.http_client)

    def icd9(self):
        """Returns an ICD-9 code.
        """
        return Icd9(self.http_client)

    def icd10(self):
        """Returns an ICD-10 code.
        """
        return Icd10(self.http_client)




class ClientError(Exception):

    """ClientError is used when the API returns an error"""

    def __init__(self, message, code):
        super(ClientError, self).__init__()
        self.message = message
        self.code = code



import urllib
import json


class RequestHandler(object):

    """RequestHandler takes care of encoding the request body into format given by options"""

    @staticmethod
    def render_key(parents):
        depth, new = 0, ''

        for x in parents:
            old = '[%s]' if depth > 0 else '%s'
            new += old % x
            depth += 1

        return new

    @staticmethod
    def urlencode(data, parents=None, pairs=None):
        if pairs is None:
            pairs = {}

        if parents is None:
            parents = []

        if isinstance(data, dict):
            for key, value in data.items():
                RequestHandler.urlencode(value, parents + [key], pairs)
        elif isinstance(data, list):
            for key, value in enumerate(data):
                RequestHandler.urlencode(value, parents + [key], pairs)
        else:
            pairs[RequestHandler.render_key(parents)] = data

        return pairs

    @staticmethod
    def set_body(request):
        typ = request['request_type'] if 'request_type' in request else 'json'

        # Encoding request body into JSON format
        if typ == 'json':
            request['data'] = json.dumps(request['data'])
            request['headers']['content-type'] = 'application/json'

        if typ == 'raw':
            if 'content-type' in request['headers']:
                del request['headers']['content-type']

        if 'request_type' in request:
            del request['request_type']

        return request



class AuthHandler(object):

    """AuthHandler takes care of devising the auth type and using it"""

    URL_SECRET = 2
    URL_TOKEN = 3

    def __init__(self, auth):
        self.auth = auth

    def get_auth_type(self):
        """Calculating the Authentication Type"""

        if 'client_id' in self.auth and 'client_secret' in self.auth:
            return self.URL_SECRET

        if 'access_token' in self.auth:
            return self.URL_TOKEN

        return -1

    def set(self, request):
        if len(self.auth.keys()) == 0:
            return request

        auth = self.get_auth_type()
        flag = False

        if auth == self.URL_SECRET:
            request = self.url_secret(request)
            flag = True

        if auth == self.URL_TOKEN:
            request = self.url_token(request)
            flag = True

        if not flag:
            raise StandardError("Unable to calculate authorization method. Please check")

        return request

    def url_secret(self, request):
        """OAUTH2 Authorization with client secret"""
        request['params']['client_id'] = self.auth['client_id']
        request['params']['client_secret'] = self.auth['client_secret']
        return request

    def url_token(self, request):
        """OAUTH2 Authorization with access token"""
        request['params']['access_token'] = self.auth['access_token']
        return request




class AccessToken(object):

    """Retrieve access token using API credentials.
    """

    def __init__(self, client):
        self.client = client

    def retrieve(self, options={}):
        """Returns an access token (required for making all other API calls).

        'oauth/token?grant_type=client_credentials' POST
        """
        body = options['body'] if 'body' in options else {}

        response = self.client.post('oauth/token?grant_type=client_credentials', body, options)

        return response




import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='aqua-io',
    version='0.1.0',
    description='Official Aqua.io API library client for Python',
    author='Michael Carroll / Aqua.io',
    author_email='michael@aqua.io',
    url='https://aqua.io',
    download_url='https://github.com/Aquaio/aqua-io-python/tarball/0.1',
    keywords=[
        'aqua',
        'aqua.io',
        'api',
        'client',
        'library',
        'ICD',
        'ICD9',
        'ICD10',
        'ICD-9',
        'ICD-10',
        'meaningful use',
        'healthcare',
        'health',
        'EHR',
        'EMR',
        'medicine',
        'medical'
    ],
    license='MIT',
    install_requires=[
        'requests >= 2.1.0'
    ],
    packages=[
        'aqua_io',
        'aqua_io.api',
        'aqua_io.error',
        'aqua_io.http_client'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



class Icd10(object):

    """Returns an ICD-10 code.
    """

    def __init__(self, client):
        self.client = client

    def top_level_codes(self, options={}):
        """Returns all top-level ICD-10 codes. Useful for helping a user navigate down the ICD-10 hierarchy to find a code.

        'codes/v1/icd10' GET
        """
        body = options['query'] if 'query' in options else {}

        response = self.client.get('codes/v1/icd10', body, options)

        return response

    def single_code(self, code_name, options={}):
        """Returns a single code matching the name, if any exists. Replace periods with hypens (e.g. 'R50-9' for 'R50.9')

        'codes/v1/icd10/:code_name' GET

        Args:
            code_name: name of code
        """
        body = options['query'] if 'query' in options else {}
        body['code_name'] = code_name

        response = self.client.get('codes/v1/icd10/' + code_name + '', body, options)

        return response

    def search_by_name(self, query, options={}):
        """Returns all codes whose name contains the search string.

        'codes/v1/icd10?q[name_cont]=:query' GET

        Args:
            query: the search query string
        """
        body = options['query'] if 'query' in options else {}
        body['query'] = query

        response = self.client.get('codes/v1/icd10?q[name_cont]=' + query + '', body, options)

        return response

    def search_by_description(self, query, options={}):
        """Returns all codes whose description contains the search string.

        'codes/v1/icd10?q[description_cont]=:query' GET

        Args:
            query: the search query string
        """
        body = options['query'] if 'query' in options else {}
        body['query'] = query

        response = self.client.get('codes/v1/icd10?q[description_cont]=' + query + '', body, options)

        return response

    def search_by_name_or_description(self, query, options={}):
        """Returns all codes whose name or description contains the search string.

        'codes/v1/icd10?q[name_or_description_cont]=:query' GET

        Args:
            query: the search query string
        """
        body = options['query'] if 'query' in options else {}
        body['query'] = query

        response = self.client.get('codes/v1/icd10?q[name_or_description_cont]=' + query + '', body, options)

        return response

